#!/bin/bash
#
# This script generates the file "gauss-lobatto.inc", which contains lists of
# Gauss-Lobatto points
#
# $Id: gauss-lobatto.sh,v 1.1 2022/06/14 08:41:14 zlb Exp $

set -e

QUADRULE_DIR="../utils"
dir="`pwd`"
cd "$QUADRULE_DIR"
make -s quadrule
cd "$dir"

# get DOF_TYPE_ORDER_MAX
order_max=`grep DOF_TYPE_ORDER_MAX ../include/phg-to-p4est.h | \
					sed -e s'/^.*MAX[ 	]*//'`
echo "DOF_TYPE_ORDER_MAX: $order_max"

{
cat <<END
/* This file is generated by gauss-lobatto.sh, don't Edit!!!
 * \$Id: gauss-lobatto.sh,v 1.1 2022/06/14 08:41:14 zlb Exp $ */

/* Note: shouldn't use quad-permu.h which works with barycentric coordinates */
#define Perm2(a)	_F(0.5)
#define Perm11(a)	_F(a),_F(1.)-(_F(a))
#define Cons2(a)	Perm2(a)
#define Cons11(a)	Perm11(a)

END
Gtab="FLOAT *Gtab[] = {NULL"
for ((np = 2; np <= order_max + 1; np++)); do
    echo 1>&2 "Processing np=$np ..."
    Gtab="$Gtab,G$np"
    echo ""
    "$QUADRULE_DIR"/quadrule 1 $((np*2-3)) $np : :1 1000 2>/dev/null | \
    "$QUADRULE_DIR"/newton 1 $((np*2-3)) 2>/dev/null | \
    awk '
	BEGIN {flag=0}
	/^static FLOAT QUAD_1D_P'$((np*2-3))'_pts/ {
	    flag=1
	    print "FLOAT G'$np'[] = {"
	    next}
	{if (flag) print; if ($0 == "};") flag=0}'
done
echo ""
echo $Gtab"};"
} >gauss-lobatto.inc
